#version 130

in vec3 worldPosition;
in vec4 clipPosition;

out vec4 outColor;


// --------------------------
// GEOMETRY EFFECT PARAMETERS

uniform vec3 CameraPosition;
uniform vec3 DiffuseLight;
uniform vec3 LightVector;
uniform bool FogEnabled;
uniform vec3 FogColor;
uniform float FogStart;
uniform float FogEnd;


// -----------------------
// WATER EFFECT PARAMETERS

uniform sampler2D ReflectionMap;
uniform sampler2D NormalMap;
uniform float RippleScale;
uniform vec2 RippleOffset;
uniform bool TransparencyEnabled;

void main()
{
	// Calculate the distance between the fragment and the camera.
	vec3 viewVector = worldPosition - CameraPosition;
	float viewDistance = length(viewVector);

	// Sample the water ripple map.
	vec2 texcoords = worldPosition.xy * RippleScale + RippleOffset;
	vec3 normal = texture(NormalMap, texcoords).rgb * 2.0 - vec3(1.0, 1.0, 1.0);

	// Calculate the specular component. 
	float specularPower = 40;
	float specularIntensity = 0.6;
	vec3 reflectionVector = normalize(2 * dot(normal, LightVector) * normal - LightVector); 
	float specular = pow(clamp(dot(reflectionVector, normalize(viewVector)), 0f, 1f), specularPower) * specularIntensity; 
	vec3 specularLight = DiffuseLight * specular;

	// Calculate the position of the fragment on screen relative to the
	// bottom left corner. These coordinates will be used for sampling
	// the reflection texture.
	vec2 screenPosition = clipPosition.xy / clipPosition.w + normal.xy * 0.03;
	screenPosition = screenPosition / 2 + vec2(0.5, 0.5);
	vec3 color = texture(ReflectionMap, screenPosition).rgb + specularLight;

	if (FogEnabled)
	{
		// Add fog to the output color.
		float fogFactor =  clamp((FogEnd - viewDistance) / (FogEnd - FogStart), 0, 1);
		color = fogFactor * color + (1.0 - fogFactor) * FogColor;
	}

	// Calculate transparency using the fresnel term.
	float fresnelTerm = 1 - pow(clamp(dot(vec3(0, 0, 1), normalize(-viewVector)), 0, 1), 1.5);

	float alpha = TransparencyEnabled ? fresnelTerm : 1.0;
	outColor = vec4(color, alpha);
}